package sita;

import org.sanskritlibrary.catalogs.ApplicationState;
import java.awt.Color;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import javax.swing.JTextArea;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.text.Caret;

public class RestorableTextArea extends JTextArea implements FocusListener, CaretListener
{
    private boolean preserveSelections = true;          // will override normal caret processing so that highlights don't disappear on a mouse click

    // The following apply only if preserveSelection is true;
    private int saveCaret = 0;                         // position of caret to restore
    private int selectionStart = 0;                    // selection start to restore
    private int selectionEnd = 0;                      // selection end to restore

    private boolean normalCaretProcessing = true;       // when this is set to false, the caretUpdate will restore the selection, instead of getting rid of it.
    private boolean doRestoreSelection = false;         // if true then getFocus will restore the selection or caret position
    private boolean ignoreCaretUpdate = false;          // internal use only

    protected ApplicationState appstate;
    protected MarkedPanel markedPanel;


    public RestorableTextArea(ApplicationState appstate, int numRows, int numCols, boolean preserveSelections)
    {
        super(numRows, numCols);
        this.preserveSelections = preserveSelections;

        this.addFocusListener(this);        // get called with we gain or lose focus

        if (preserveSelections)
            this.addCaretListener(this);    // preserve selection or caret position on initial mouse click

        this.appstate = appstate;
    }

    public void resetCaretProcessing()
    {
        saveCaret = 0;                         // position of caret to restore
        selectionStart = 0;                    // selection start to restore
        selectionEnd = 0;                      // selection end to restore

        normalCaretProcessing = true;       // when this is set to false, the caretUpdate will restore the selection, instead of getting rid of it.
        doRestoreSelection = false;         // if true then getFocus will restore the selection or caret position
        ignoreCaretUpdate = false;          // internal use only
    }

    // This should be called when displaying an existing annotation. It will insure that any selection will not be destroyed by a mouse click
    public void rememberSelection(int selectionStart, int selectionEnd)
    {
        if (preserveSelections)
        {
            this.selectionStart = selectionStart;
            this.selectionEnd = selectionEnd;
            doRestoreSelection = true;      // selection will be restored on focusGained instead of caretUpdate
        }
    }

    // Wrapper for normal setCaretPosition that turns off any special caret process.
    public void _setCaretPosition(int n)
    {
        if (n >= 0)
        {
            normalCaretProcessing = true;
            setCaretPosition(n);
        }
        else
            java.awt.Toolkit.getDefaultToolkit().beep();
    }

    public void setMarkedPanel(MarkedPanel markedPanel)
    {
        this.markedPanel = markedPanel;
    }

    public void focusGained(FocusEvent e)       // FocusListener
    {
        if (markedPanel != null)
            markedPanel.setColor(Color.red);

        if (doRestoreSelection)                 // need to restore after dialog closed, i.e., there will be no mouse click that will cause the normal caretUpdate to be called.
        {
            restoreSelection();                 // put back the selection
            doRestoreSelection = false;         // force this to be set to false again.
        }
        else                                    // normal processing, a mouse click caused the focus and caretUpdate will restore the selection
        {
            Caret c = getCaret();
            c.setVisible(true);
        }
    }

    public void focusLost(FocusEvent e)         // FocusListener
    {
        if (markedPanel != null)
            markedPanel.setColor(Color.white);

        // If rememberSelection was not called, save everything we need to restore things to exactly as they are now when focus is regained
        if (preserveSelections)
        {
            Caret c = getCaret();
            c.setVisible(true);                 // keep cursor visible at all times
            c.setSelectionVisible(true);
            selectionStart = getSelectionStart();
            selectionEnd = getSelectionEnd();
            saveCaret = getCaretPosition();
            normalCaretProcessing = false;      // Signal that on next caret update, we want to handle things
        }
    }

    public void setDoRestoreSelection(boolean b)
    {
        doRestoreSelection = b;
    }

    public boolean getDoRestoreSelection()
    {
        return doRestoreSelection;
    }

    public void caretUpdate(CaretEvent e)       // CaretListener
    {
        if (ignoreCaretUpdate)
            return;

        if (!normalCaretProcessing)
        {
            restoreSelection();
            normalCaretProcessing = true;
        }
    }

    private void restoreSelection()
    {
        ignoreCaretUpdate = true;
        Caret c = getCaret();
        c.setVisible(true);
        int len = getText().length();
        if (selectionStart != selectionEnd)
        {
            if (selectionEnd <= len)
            {
                setCaretPosition(selectionStart);
                moveCaretPosition(selectionEnd);
            }
        }
        else if (saveCaret <= len)
            setCaretPosition(saveCaret);
        ignoreCaretUpdate = false;
    }
}
